@@ -10,8 +10,9 @@
 #ifndef _SPI_IMPL_H_
 #define _SPI_IMPL_H_
 
-#include <orbit/orbit.h>
-#include "Accessibility.h"
+#include <libspi/accessibleeventlistener.h>
+#include <libspi/keystrokelistener.h>
+#include <libspi/Accessibility.h>
 
 typedef unsigned int boolean;
 
@@ -32,4 +33,7 @@ typedef CORBA_Object AccessibleValue;
 typedef CORBA_Object AccessibilityRegistry;
 typedef CORBA_Object GenericInterface;
 
+typedef SpiKeystrokeListener AccessibleKeystrokeListener;
+typedef SpiEventListener AccessibleEventListener;
+
 #endif
