@@ -20,59 +20,33 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/*
- * accessible.c: test for accessibility implementation
- *
- */
+/* accessible.c: the core of the accessibility implementation */
+
 #include <config.h>
-#include <bonobo/Bonobo.h>
-
 #include <stdio.h>
+#include <libspi/libspi.h>
 
-/*
- * This pulls the CORBA definitions for the "Accessibility::Accessible" server
- */
-#include <libspi/Accessibility.h>
+/* Our parent Gtk object type  */
+#define PARENT_TYPE BONOBO_TYPE_OBJECT
 
-/*
- * This pulls the definition for the BonoboObject (Gtk Type)
- */
-#include "accessible.h"
-#include "component.h"
-#include "editabletext.h"
-#include "hyperlink.h"
-#include "hypertext.h"
-#include "image.h"
-#include "selection.h"
-#include "table.h"
-#include "text.h"
-#include "value.h"
-
-/*
- * Our parent Gtk object type
- */
-#define PARENT_TYPE BONOBO_OBJECT_TYPE
-
-/*
- * A pointer to our parent object class
- */
-static GObjectClass *accessible_parent_class;
+/* A pointer to our parent object class */
+static GObjectClass *spi_accessible_parent_class;
 
 /*
  * Implemented GObject::finalize
  */
 static void
-accessible_object_finalize (GObject *object)
+spi_accessible_object_finalize (GObject *object)
 {
-        Accessible *accessible = ACCESSIBLE (object);
+        SpiAccessible *accessible = SPI_ACCESSIBLE (object);
 
-        printf("accessible_object_finalize called\n");
+        printf("spi_accessible_object_finalize called\n");
 	ATK_OBJECT (accessible->atko); /* assertion */
         g_object_unref (G_OBJECT(accessible->atko));
         accessible->atko = NULL;
 
         printf("atko freed, calling parent finalize\n");
-        accessible_parent_class->finalize (object);
+        spi_accessible_parent_class->finalize (object);
 }
 
 /*
@@ -83,7 +57,7 @@ impl_accessibility_accessible_get_name (PortableServer_Servant servant,
                                         CORBA_Environment     *ev)
 {
   CORBA_char * retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   retval = (CORBA_char *) atk_object_get_name (accessible->atko);
   if (retval )
     retval = CORBA_string_dup (retval);
@@ -101,9 +75,9 @@ impl_accessibility_accessible_set_name (PortableServer_Servant servant,
                                         const CORBA_char      *name,
                                         CORBA_Environment     *ev)
 {
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   atk_object_set_name (accessible->atko, name);
-  printf ("Accessible set_name called: %s\n", name);
+  printf ("SpiAccessible set_name called: %s\n", name);
 }
 
 /*
@@ -114,8 +88,12 @@ impl_accessibility_accessible_get_description (PortableServer_Servant servant,
                                                CORBA_Environment     *ev)
 {
   CORBA_char * retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   retval = CORBA_string_dup (atk_object_get_description (accessible->atko));
+  if (retval )
+    retval = CORBA_string_dup (retval);
+  else
+    retval = CORBA_string_dup ("");
 
   return retval;
 }
@@ -128,9 +106,9 @@ impl_accessibility_accessible_set_description (PortableServer_Servant servant,
                                                const CORBA_char      *name,
                                                CORBA_Environment     *ev)
 {
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   atk_object_set_description (accessible->atko, name);
-  printf ("Accessible set_description called: %s\n", name);
+  printf ("SpiAccessible set_description called: %s\n", name);
 }
 
 /*
@@ -141,12 +119,15 @@ impl_accessibility_accessible_get_parent (PortableServer_Servant servant,
                                           CORBA_Environment     *ev)
 {
   Accessibility_Accessible retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
-  AtkObject *parent;
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
+  AtkObject     *parent;
+
   parent = atk_object_get_parent (accessible->atko);
-  retval = BONOBO_OBJREF (accessible_new (parent));
-  printf ("Accessible get_parent called\n");
-  return CORBA_Object_duplicate (retval, &ev);
+  retval = BONOBO_OBJREF (spi_accessible_new (parent));
+
+  printf ("SpiAccessible get_parent called\n");
+
+  return CORBA_Object_duplicate (retval, ev);
 }
 
 /*
@@ -157,9 +138,9 @@ impl_accessibility_accessible_get_index_in_parent (PortableServer_Servant servan
                                                    CORBA_Environment     *ev)
 {
   CORBA_long retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   retval = (CORBA_long) atk_object_get_index_in_parent (accessible->atko);
-  printf ("Accessible get_index_in_parent called\n");
+  printf ("SpiAccessible get_index_in_parent called\n");
   return retval;
 }
 
@@ -171,9 +152,9 @@ impl_accessibility_accessible_get_child_count (PortableServer_Servant servant,
                                                CORBA_Environment     *ev)
 {
   CORBA_long retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   retval = (CORBA_long) atk_object_get_n_accessible_children (accessible->atko);
-  printf ("Accessible get_childCount called: %d\n", (int) retval);
+  printf ("SpiAccessible get_childCount called: %d\n", (int) retval);
   return retval;
 }
 
@@ -186,11 +167,11 @@ impl_accessibility_accessible_get_child_at_index (PortableServer_Servant servant
                                                   CORBA_Environment     *ev)
 {
   Accessibility_Accessible retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   AtkObject *child = atk_object_ref_accessible_child (accessible->atko, (gint) index);
-  retval = BONOBO_OBJREF (accessible_new (child));
-  printf ("Accessible get_child_at_index called.\n");
-  return CORBA_Object_duplicate (retval, &ev);
+  retval = BONOBO_OBJREF (spi_accessible_new (child));
+  printf ("SpiAccessible get_child_at_index called.\n");
+  return CORBA_Object_duplicate (retval, ev);
 }
 
 /*
@@ -201,10 +182,10 @@ impl_accessibility_accessible_get_state (PortableServer_Servant servant,
 					 CORBA_Environment     *ev)
 {
   Accessibility_StateSet retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
-  AtkStateSet *state = atk_object_ref_state_set (accessible->atko);
+/*  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
+    AtkStateSet *state = atk_object_ref_state_set (accessible->atko); */
   retval = CORBA_OBJECT_NIL;
-  printf ("Accessible get_state.\n");
+  printf ("SpiAccessible get_state.\n");
   /* TODO: implement the bonobo stateset class */
   return (Accessibility_StateSet) retval;
 }
@@ -217,14 +198,23 @@ impl_accessibility_accessible_get_relation_set (PortableServer_Servant servant,
 						CORBA_Environment     *ev)
 {
   Accessibility_RelationSet *retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  gint n_relations;
+  gint i;
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   AtkRelationSet *relation_set = atk_object_ref_relation_set (accessible->atko);
+  n_relations = atk_relation_set_get_n_relations (relation_set);
   retval = CORBA_sequence_Accessibility_Relation__alloc ();
-  /*
-   *  TODO: fill the sequence with relation set objects, themselves
-   *  initialized from the AtkRelation object in the AtkRelationSet.
-   */
-  printf ("Accessible get_relation_set.\n");
+  CORBA_sequence_Accessibility_Relation_allocbuf (n_relations);
+	  
+  for (i=0; i<n_relations; ++i)
+    {
+      retval->_buffer[i] =
+	      CORBA_Object_duplicate (bonobo_object_corba_objref (
+		      BONOBO_OBJECT (spi_relation_new (atk_relation_set_get_relation (relation_set, i)))),
+				      ev);
+    }
+  
+  printf ("SpiAccessible get_relation_set.\n");
   return retval;
 }
 
@@ -236,21 +226,21 @@ impl_accessibility_accessible_get_role (PortableServer_Servant servant,
 					CORBA_Environment     *ev)
 {
   Accessibility_Role retval;
-  Accessible *accessible = ACCESSIBLE (bonobo_object_from_servant (servant));
+  SpiAccessible *accessible = SPI_ACCESSIBLE (bonobo_object_from_servant (servant));
   AtkRole role = atk_object_get_role (accessible->atko);
-  retval = role;
-  printf ("Accessible get_role.\n");
+  retval = role; /* relies on ability to cast these back and forth */
+  printf ("SpiAccessible get_role.\n");
   return (Accessibility_Role) retval;
 }
 
 static void
-accessible_class_init (AccessibleClass *klass)
+spi_accessible_class_init (SpiAccessibleClass *klass)
 {
         GObjectClass * object_class = (GObjectClass *) klass;
         POA_Accessibility_Accessible__epv *epv = &klass->epv;
-        accessible_parent_class = g_type_class_peek_parent (klass);
+        spi_accessible_parent_class = g_type_class_peek_parent (klass);
 
-        object_class->finalize = accessible_object_finalize;
+        object_class->finalize = spi_accessible_object_finalize;
 
         epv->_get_name = impl_accessibility_accessible_get_name;
         epv->_set_name = impl_accessibility_accessible_set_name;
@@ -268,114 +258,78 @@ accessible_class_init (AccessibleClass *klass)
 }
 
 static void
-accessible_init (Accessible *accessible)
+spi_accessible_init (SpiAccessible *accessible)
 {
 }
 
-GType
-accessible_get_type (void)
+BONOBO_TYPE_FUNC_FULL (SpiAccessible,
+		       Accessibility_Accessible,
+		       PARENT_TYPE,
+		       spi_accessible);
+
+SpiAccessible *
+spi_accessible_new (AtkObject *o)
 {
-        static GType type = 0;
-
-        if (!type) {
-                static const GTypeInfo tinfo = {
-                        sizeof (AccessibleClass),
-                        (GBaseInitFunc) NULL,
-                        (GBaseFinalizeFunc) NULL,
-                        (GClassInitFunc) accessible_class_init,
-                        (GClassFinalizeFunc) NULL,
-                        NULL, /* class data */
-                        sizeof (Accessible),
-                        0, /* n preallocs */
-                        (GInstanceInitFunc) accessible_init,
-                        NULL /* value table */
-                };
-                /*
-                 * Bonobo_type_unique auto-generates a load of
-                 * CORBA structures for us. All derived types must
-                 * use bonobo_type_unique.
-                 */
-                type = bonobo_type_unique (
-                        PARENT_TYPE,
-                        POA_Accessibility_Accessible__init,
-                        NULL,
-                        G_STRUCT_OFFSET (AccessibleClass, epv),
-                        &tinfo,
-                        "Accessible");
-        }
-
-        return type;
-}
-
-Accessible *
-accessible_new (AtkObject *o)
-{
-    Accessible *retval =
-               ACCESSIBLE (g_object_new (accessible_get_type (), NULL));
+    SpiAccessible *retval = g_object_new (SPI_ACCESSIBLE_TYPE, NULL);
     CORBA_Environment ev;
     CORBA_exception_init (&ev);
     g_object_ref (o);
     retval->atko = ATK_OBJECT (o);
 
-    /*
-     * TODO: add interface containers/constructors for EDITABLE_TEXT, HYPERTEXT,
-     *  IMAGE, SELECTION, TABLE, TEXT, VALUE.
-     */
-
-    /* add appropriate ATK interfaces */
+    /* aggregate appropriate SPI interfaces based on ATK interfaces */
 
     if (ATK_IS_ACTION (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT (action_interface_new (o)));
+                                     BONOBO_OBJECT (spi_action_interface_new (o)));
       }
 
     if (ATK_IS_COMPONENT (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT (component_interface_new (o)));
+                                     BONOBO_OBJECT (spi_component_interface_new (o)));
       }
 
     if (ATK_IS_EDITABLE_TEXT (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT(editable_text_interface_new (o)));
+                                     BONOBO_OBJECT(spi_editable_text_interface_new (o)));
       }
 
     else if (ATK_IS_HYPERTEXT (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT (hypertext_interface_new (o)));
+                                     BONOBO_OBJECT (spi_hypertext_interface_new (o)));
       }
 
     else if (ATK_IS_TEXT (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT (text_interface_new (o)));
+                                     BONOBO_OBJECT (spi_text_interface_new (o)));
       }
 
     if (ATK_IS_IMAGE (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT (image_interface_new (o)));
+                                     BONOBO_OBJECT (spi_image_interface_new (o)));
       }
 
     if (ATK_IS_SELECTION (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT (selection_interface_new (o)));
+                                     BONOBO_OBJECT (spi_selection_interface_new (o)));
       }
 
     if (ATK_IS_TABLE (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT (table_interface_new (o)));
+                                     BONOBO_OBJECT (spi_table_interface_new (o)));
       }
 
     if (ATK_IS_VALUE (o))
       {
         bonobo_object_add_interface (bonobo_object (retval),
-                                     BONOBO_OBJECT (value_interface_new (o)));
+                                     BONOBO_OBJECT (spi_value_interface_new (o)));
       }
 
     return retval;
