@@ -17,26 +17,23 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef SELECTION_H_
-#define SELECTION_H_
-
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+#ifndef SPI_SELECTION_H_
+#define SPI_SELECTION_H_
 
 #include <bonobo/bonobo-object.h>
 #include <atk/atk.h>
 #include <libspi/Accessibility.h>
 
-#define SELECTION_TYPE        (selection_get_type ())
-#define SELECTION(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), SELECTION_TYPE, Selection))
-#define SELECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), SELECTION_TYPE, SelectionClass))
-#define IS_SELECTION(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SELECTION_TYPE))
-#define IS_SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SELECTION_TYPE))
+G_BEGIN_DECLS
 
-typedef struct _Selection Selection;
-typedef struct _SelectionClass SelectionClass;
+#define SPI_SELECTION_TYPE        (spi_selection_get_type ())
+#define SPI_SELECTION(obj)          (G_TYPE_CHECK_INSTANCE_CAST ((obj), SPI_SELECTION_TYPE, SpiSelection))
+#define SPI_SELECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), SPI_SELECTION_TYPE, SpiSelectionClass))
+#define IS_SPI_SELECTION(obj)       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), SPI_SELECTION_TYPE))
+#define IS_SPI_SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), SPI_SELECTION_TYPE))
+
+typedef struct _Selection SpiSelection;
+typedef struct _SelectionClass SpiSelectionClass;
 
 struct _Selection {
   BonoboObject parent;
@@ -49,13 +46,11 @@ struct _SelectionClass {
 };
 
 GType
-selection_get_type   (void);
+spi_selection_get_type   (void);
 
-Selection *
-selection_interface_new       (AtkObject *obj);
+SpiSelection *
+spi_selection_interface_new       (AtkObject *obj);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
-#endif /* SELECTION_H_ */
+#endif /* SPI_SELECTION_H_ */
